from typing import Dict, Literal, Optional

import aiohttp
import requests

from axew import (
    InvalidParams,
    Entry,
    BASE_URL,
    EntryNotFound,
    MAX_SIZE,
    ValidationError,
    BadRequest,
    BaseAxewException,
)


class AxewClient:
    def __init__(self):
        self.cache: Dict[str, Entry] = {}

    @staticmethod
    def create_entry(data: Dict) -> Entry:
        return Entry(**data)

    @staticmethod
    def validate_entry(
        code: str,
        error: Optional[str] = "",
        name: Optional[str] = "",
        description: Optional[str] = "",
    ) -> None:
        """
        Validates the given items are within size limits.

        Parameters
        ----------
        code: str
            ...
        error: Optional[str]
            ...
        name: Optional[str]
            ...
        description: Optional[str]
            ...

        Returns
        -------
        None
            This entry is valid

        Raises
        ------
        ValidationError
            The given item is out of size
        """
        if len(code) > MAX_SIZE["code"]:
            raise ValidationError("code", MAX_SIZE["code"])

        if error and len(error) > MAX_SIZE["error"]:
            raise ValidationError("error", MAX_SIZE["error"])

        if name and len(name) > MAX_SIZE["name"]:
            raise ValidationError("name", MAX_SIZE["name"])

        if description and len(description) > MAX_SIZE["description"]:
            raise ValidationError("description", MAX_SIZE["description"])

    def create_paste(
        self,
        *,
        code: str,
        error: str = "",
        name: str = "",
        description: str = "",
    ) -> Entry:
        """Creates a new paste

        Parameters
        ----------
        code : str
            The code portion of the paste
        error : str, optional
            The error portion of the paste
        name : str, optional
            What to name this entry
        description : str, optional
            A short description of this entry

        Returns
        -------
        Entry
            The created entry

        Raises
        ------
        InvalidParams
            Invalid provided arguments
        ValidationError
            The given arguments were too long
        """
        self.validate_entry(code, error, name, description)
        data = {"code": code}
        if error:
            data["error"] = error

        if name:
            data["name"] = name

        if description:
            data["description"] = description

        r = requests.post(f"{BASE_URL}/v1/entries/", json=data)
        return_data = r.json()
        if r.status_code == 400:
            raise BadRequest(return_data["message"])

        elif r.status_code == 422:
            raise BaseAxewException(str(return_data))

        entry = self.create_entry(return_data)
        self.cache[entry.uuid] = entry

        return entry

    def get_paste(self, uuid: str) -> Entry:
        """Returns the given Entry for a uuid

        Parameters
        ----------
        uuid : str
            The uuid for the entry you wish to fetch

        Returns
        -------
        Entry
            The entry for said uuid

        Raises
        ------
        EntryNotFound
            Couldn't find an entry with that uuid
        """
        if uuid in self.cache:
            return self.cache[uuid]

        r = requests.get(f"{BASE_URL}/v1/entries/{uuid}/")
        if r.status_code != 200:
            raise EntryNotFound

        return_data = r.json()
        entry: Entry = self.create_entry(return_data)
        return entry



import datetime
from dataclasses import dataclass
from typing import Dict

BASE_URL = "https://api.arcanebin.com"
MAX_SIZE = {
    "name": 200,
    "description": 2500,
    "code": 15000,
    "error": 5000,
}


@dataclass
class Entry:
    uuid: str = None
    code: str = None
    error: str = None
    name: str = None
    description: str = None
    created_by: Dict = None
    created_at: datetime.datetime = None

    def resolve_url(self):
        return BASE_URL + "/view/" + self.uuid



class BaseAxewException(Exception):
    """The base Axew exception."""

    def __init__(self, *args):
        if args:
            self.message = args[0]
        else:
            self.message = self.__doc__

    def __str__(self):
        return self.message


class EntryNotFound(BaseAxewException):
    """Failed to find the given entry."""


class InvalidParams(BaseAxewException):
    """An entry requires at-least code."""


class ValidationError(BaseAxewException):
    """The given arguments were too long."""

    def __init__(self, var, size):
        super().__init__()
        self.message += f" {var} needs to be shorter then {size}."


class BadRequest(BaseAxewException):
    """Something went wrong."""

    def __init__(self, message):
        super().__init__()
        self.message = message



from .entry import Entry, BASE_URL, MAX_SIZE
from .exceptions import *
from .client import AxewClient

__version__ = "1.0.7"

import logging
from collections import namedtuple

logging.getLogger(__name__).addHandler(logging.NullHandler())
VersionInfo = namedtuple("VersionInfo", "major minor micro releaselevel serial")
version_info = VersionInfo(major=1, minor=0, micro=7, releaselevel="final", serial=0)



import re

import setuptools

with open("readme.md", "r") as fh:
    long_description = fh.read()

_version_regex = (
    r"^__version__ = ('|\")((?:[0-9]+\.)*[0-9]+(?:\.?([a-z]+)(?:\.?[0-9])?)?)\1$"
)

with open("axew/__init__.py") as stream:
    match = re.search(_version_regex, stream.read(), re.MULTILINE)

version = match.group(2)

setuptools.setup(
    name="axew",
    version=version,
    author="Menudocs",
    author_email="contact@menudocs.org",
    description="Python bindings for Arcane",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/MenuDocs/Axew",
    packages=setuptools.find_packages(),
    install_requires=["requests", "aiohttp"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
    ],
    python_requires=">=3.8",
)
